#include <ui.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef _WIN32
#include <Windows.h>
#include <fileapi.h>
#include <stdio.h> 
#else
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#endif
#include <iostream>
#include <thread>
#include <future>
#include <rew/decoder/decoder.h>
#include <rew/common/named_raw_file.h>
#include <rew/decoder/physical_audio_source.h>
#include "rew/encoder/audio_sink.h"
#include "rew/encoder/wav_writer.h"

#ifdef _MSC_VER
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif

static rew::PhysicalAudioSource* gSource = nullptr;

///=====================================================================================================================
std::string getFullPath(const std::string& path) {
#ifdef _WIN32
    LPSTR* lppPart = { nullptr };
    std::string buffer(4096, '\0');
    if (GetFullPathName(path.c_str(), buffer.size(), &buffer[0], lppPart) == 0) {
        throw std::runtime_error("Failed to get full path");
    }
#else
    std::string buffer(PATH_MAX, '\0');
    realpath(path.c_str(), &buffer[0]);
#endif
    return std::string(&buffer[0]);
}

///=====================================================================================================================
#ifdef _WIN32
BOOL WINAPI ctrlHandler(const DWORD fdwCtrlType) {
    switch (fdwCtrlType) {
        case CTRL_C_EVENT:
        case CTRL_CLOSE_EVENT:
        case CTRL_BREAK_EVENT:
        case CTRL_LOGOFF_EVENT:
        case CTRL_SHUTDOWN_EVENT:
            if (gSource) {
                gSource->close();
            }
            return FALSE;
        default:
            return FALSE;
    }
}
#endif

///=====================================================================================================================
size_t getFileSize(const std::string& path) {
    struct stat info = { 0 };
    if (stat(path.c_str(), &info) != 0)return false;
    return static_cast<size_t>(info.st_size);
}

///=====================================================================================================================
bool folderIsValid(const std::string& path) {
    struct stat info = { 0 };
    if (stat(path.c_str(), &info) != 0)return false;
    else if (info.st_mode & S_IFDIR) return true;
    else return false;
}

///=====================================================================================================================
bool pathIsValid(const std::string& path) {
    struct stat info = { 0 };
    return stat(path.c_str(), &info) == 0;
}

///=====================================================================================================================
class CustomSink: public rew::Input<rew::NamedRawFile> {
public:
    CustomSink(const std::string& destination):destination(destination) {
        
    }
    virtual ~CustomSink() = default;

    void process(const rew::NamedRawFile* data, const size_t length) override {
        for (size_t i = 0; i < length; i++) {
            const auto& namedFile = data[i];

            auto terminator = namedFile.length;
            for (size_t i = 0; i < namedFile.length; i++) {
                if (namedFile.data[i] == '\0') {
                    terminator = i;
                    break;
                }
            }

            if (terminator != namedFile.length) {
                const auto sptr = reinterpret_cast<const char*>(namedFile.data.get());
                const auto name = std::string(sptr, terminator);

                const auto contents = reinterpret_cast<const char*>(namedFile.data.get() + terminator + 1);
                const auto total = namedFile.length - terminator - 1;
                std::string path = destination + DELIMITER + name;

                std::cout << "Writing file: " << path << " of size: " << total << " bytes!" << std::endl;
                std::fstream file(path, std::ios::out | std::ios::binary);
                if (!file) {
                    std::cerr << "Failed to open file: " << path << " for writing!" << std::endl;
                }
                else {
                    file.write(contents, total);
                }
            }
            else {
                std::cerr << "File index: " << namedFile.index << " successfully inflated but contains bad filename!" << std::endl;
            }
        }
    }

private:
    std::string destination;
};

///=====================================================================================================================
int main (const int argc, char* argv[]) {
#ifdef _WIN32
    SetConsoleCtrlHandler(ctrlHandler, TRUE);
#endif

    try {
        return EXIT_SUCCESS;
    }
    catch (const std::exception& e) {
        std::cerr << e.what() << std::endl;
        return EXIT_FAILURE;
    }
}
