#include <ui.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef _WIN32
#include <Windows.h>
#include <fileapi.h>
#else
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#endif
#include <iostream>
#include "folder.h"
#include <rew/encoder/encoder.h>
#include <rew/encoder/physical_audio_sink.h>

#ifdef _MSC_VER
#pragma comment(linker,"/manifestdependency:\"type='win32' name='Microsoft.Windows.Common-Controls' version='6.0.0.0' processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")
#endif

///=====================================================================================================================
std::string getFullPath(const std::string& path) {
#ifdef _WIN32
    LPSTR* lppPart = {nullptr};
    std::string buffer(4096, '\0');
    if (GetFullPathName(path.c_str(), buffer.size(), &buffer[0], lppPart) == 0) {
        throw std::runtime_error("Failed to get full path");
    }
#else
    std::string buffer(PATH_MAX, '\0');
    realpath(path.c_str(), &buffer[0]);
#endif
    return std::string(&buffer[0]);
}

///=====================================================================================================================
size_t getFileSize(const std::string& path) {
    struct stat info = {0};
    if (stat(path.c_str(), &info) != 0)return false;
    return static_cast<size_t>(info.st_size);
}

///=====================================================================================================================
bool folderIsValid(const std::string& path) {
    struct stat info = {0};
    if (stat(path.c_str(), &info) != 0)return false;
    else if (info.st_mode & S_IFDIR) return true;
    else return false;
}

///=====================================================================================================================
bool pathIsValid(const std::string& path) {
    struct stat info = {0};
    return stat(path.c_str(), &info) == 0;
}

///=====================================================================================================================
std::vector<std::string> recursivelyParseFolder(const std::string& path) {
    std::vector<std::string> files;

    //std::cout << "Entering:   " << path << std::endl;
    auto folder = rew::Folder(path);
    std::string file;

    while (folder.next(file)) {
        if (file == "." || file == "..") continue;

        const auto filePath = path + DELIMITER + file;

        if (folderIsValid(filePath)) {
            const auto b = recursivelyParseFolder(filePath);
            files.insert(std::end(files), std::begin(b), std::end(b));
        }
        else {
            //std::cout << "Collecting: " << filePath << std::endl;
            files.push_back(filePath);
        }
    }

    return files;
}

///=====================================================================================================================
class DirectoryTraverser : public rew::Output<rew::FileName> {
public:
    DirectoryTraverser() = default;
    virtual ~DirectoryTraverser() = default;

    void process(const std::string& path) {
        // Collect all files and forward them down the pipeline
        const auto files = recursivelyParseFolder(path, path);
        this->forward(&files[0], files.size());
    }

private:
    std::vector<rew::FileName> recursivelyParseFolder(const std::string& base, const std::string& path) {
        std::vector<rew::FileName> files;

        auto folder = rew::Folder(path);
        std::string file;

        while (folder.next(file)) {
            if (file == "." || file == "..") continue;

            const auto filePath = path + DELIMITER + file;

            if (folderIsValid(filePath)) {
                const auto b = recursivelyParseFolder(base, filePath);
                files.insert(std::end(files), std::begin(b), std::end(b));
            }
            else {
                const auto name = filePath.substr(base.size());
                files.push_back({filePath, name});
            }
        }

        return files;
    }
};

///=====================================================================================================================
int main(const int argc, char* argv[]) {
    try {
        

        return EXIT_SUCCESS;
    }
    catch (const std::exception& e) {
        std::cerr << e.what() << std::endl;
        return EXIT_FAILURE;
    }
}
